# Copyright (c) 2008-2013, Andrew Walker
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
cimport cython

cdef extern from "dubins.h":
    ctypedef struct DubinsPath:
        pass
    ctypedef int (*DubinsPathSamplingCallback)(double q[3], double t, void* user_data)
    cdef int dubins_init( double q0[3], double q1[3], double rho, DubinsPath* path )
    int dubins_path_sample_many( DubinsPath* path, DubinsPathSamplingCallback cb, double stepSize, void* user_data )

cdef int callback(double q[3], double t, void* f):
    """This is the c- proxy callback that delegates back to Python
    """
    qn = (q[0], q[1], q[2])
    return (<object>f)(qn, t)

def sample_dubins_path( q0, q1, rho, step_size ):
    """Discrete sampling of a Dubin's path

    Parameters
    ----------
    q0 : array-like shape = (3,), dtype = float
        initial configuration (x0, y0, theta0)
    q1 : array-like shape = (3,), dtype = float
        final configuration (x1, y1, theta1)
    rho : float
        equivalent to turning radius (forward velocity / angular rate)
    step_size : float
        sampling interval along the path

    Notes
    -----
    theta = 0 is along the line x = 0
    """
    cdef double _q0[3]
    cdef double _q1[3]
    for i in [0, 1, 2]:
        _q0[i] = q0[i]
        _q1[i] = q1[i]
    cdef DubinsPath path
    dubins_init(_q0, _q1, rho, cython.address(path))
    qs = []
    ts = []
    def f(q, t):
        qs.append(q)
        ts.append(t)
        return 0
    dubins_path_sample_many(cython.address(path), callback, step_size, <void*>f)
    return qs, ts



import os
from distutils.core import setup
from distutils.extension import Extension
try:
    from Cython.Distutils import build_ext
except:
    use_cython = False
else:
    use_cython = True

cmdclass = {}
ext_modules = []

if use_cython:
    ext_modules = [
        Extension("dubins",
            ["dubins/src/dubins.c", "dubins/dubins.pyx"],
            include_dirs = ["dubins/include"]
        )
    ]
    cmdclass.update({ 'build_ext' : build_ext })
else:
    ext_modules = [
        Extension("dubins",
            ["dubins/src/dubins.c", "dubins/dubins.c"],
            include_dirs = ["dubins/include"]
        )
    ]

def read(filename):
    path = os.path.join(os.path.dirname(__file__), filename)
    contents = open(path).read()
    return contents


setup(
    name         = "dubins",
    version      = "0.8.1",
    description  = "Code to generate and manipulate dubins curves",
    long_description = read('README.rst'),
    author       = "Andrew Walker",
    author_email = "walker.ab@gmail.com",
    url          = "http://github.com/AndrewWalker/pydubins",
    classifiers  = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        # TODO - further testing required to show which platforms
        #        will work correctly, but any changes should be
        #        fairly minor
        'Operating System :: POSIX :: Linux',
        'Topic :: Scientific/Engineering :: Mathematics',
    ],
    cmdclass = cmdclass,
    ext_modules  = ext_modules,
)

